@@ -6,61 +6,49 @@ dicom2nifti
 """
 
 from __future__ import print_function
+import dicom2nifti.patch_pydicom_encodings
+from dicom2nifti.exceptions import ConversionError
+
+dicom2nifti.patch_pydicom_encodings.apply()
 
 import itertools
 import os
 from math import pow
 
+import logging
 import nibabel
 import numpy
-from dicom.tag import Tag
+
+from pydicom.tag import Tag
+
 from six import string_types
 
 import dicom2nifti.common as common
 import dicom2nifti.convert_generic as convert_generic
 
-
-def is_ge(dicom_input):
-    """
-    Use this function to detect if a dicom series is a GE dataset
-    :param dicom_input: list with dicom objects
-    """
-    # read dicom header
-    header = dicom_input[0]
-
-    if 'Manufacturer' not in header or 'Modality' not in header:
-        return False  # we try generic conversion in these cases
-
-    # check if manufacturer is GE
-    if 'GE MEDICAL SYSTEMS' not in header.Manufacturer.upper():
-        return False
-
-    # check if Modality is mr
-    if header.Modality.upper() != 'MR':
-        return False
-
-    return True
+logger = logging.getLogger(__name__)
 
 
-def dicom_to_nifti(dicom_input, output_file):
+def dicom_to_nifti(dicom_input, output_file=None):
     """
     This is the main dicom to nifti conversion fuction for ge images.
     As input ge images are required. It will then determine the type of images and do the correct conversion
 
     Examples: See unit test
+
     :param output_file: the filepath to the output nifti file
     :param dicom_input: list with dicom objects
     """
-    assert is_ge(dicom_input)
+    assert common.is_ge(dicom_input)
 
-    print('Reading and sorting dicom files')
+    logger.info('Reading and sorting dicom files')
     grouped_dicoms = _get_grouped_dicoms(dicom_input)
 
     if _is_4d(grouped_dicoms):
-        print('Found sequence type: 4D')
+        logger.info('Found sequence type: 4D')
         return _4d_to_nifti(grouped_dicoms, output_file)
 
-    print('Assuming anatomical data')
+    logger.info('Assuming anatomical data')
     return convert_generic.dicom_to_nifti(dicom_input, output_file)
 
 
@@ -73,15 +61,11 @@ def _is_4d(grouped_dicoms):
     # read dicom header
     header = grouped_dicoms[0][0]
 
-    # check if the dicom contains stack information
-    if Tag(0x0020, 0x9056) not in header or Tag(0x0020, 0x9057) not in header:
-        return False
-
     # check if contains multiple stacks
-    if len(grouped_dicoms) <= 1:
-        return False
+    if len(grouped_dicoms) > 1:
+        return True
 
-    return True
+    return False
 
 
 def _is_diffusion_imaging(grouped_dicoms):
@@ -110,36 +94,45 @@ def _4d_to_nifti(grouped_dicoms, output_file):
     """
 
     # Create mosaic block
-    print('Creating data block')
+    logger.info('Creating data block')
     full_block = _get_full_block(grouped_dicoms)
 
-    print('Creating affine')
+    logger.info('Creating affine')
     # Create the nifti header info
     affine = common.create_affine(grouped_dicoms[0])
 
-    print('Creating nifti')
+    logger.info('Creating nifti')
     # Convert to nifti
-    img = nibabel.Nifti1Image(full_block, affine)
-    common.set_tr_te(img, float(grouped_dicoms[0][0].RepetitionTime),
+    nii_image = nibabel.Nifti1Image(full_block, affine)
+    common.set_tr_te(nii_image, float(grouped_dicoms[0][0].RepetitionTime),
                      float(grouped_dicoms[0][0].EchoTime))
-    print('Saving nifti to disk %s' % output_file)
+    logger.info('Saving nifti to disk %s' % output_file)
     # Save to disk
-    img.to_filename(output_file)
+    if output_file is not None:
+        nii_image.to_filename(output_file)
 
     if _is_diffusion_imaging(grouped_dicoms):
+        bval_file = None
+        bvec_file = None
         # Create the bval en bevec files
-        base_path = os.path.dirname(output_file)
-        base_name = os.path.splitext(os.path.splitext(os.path.basename(output_file))[0])[0]
+        if output_file is not None:
+            base_path = os.path.dirname(output_file)
+            base_name = os.path.splitext(os.path.splitext(os.path.basename(output_file))[0])[0]
 
-        print('Creating bval en bvec files')
-        bval_file = '%s/%s.bval' % (base_path, base_name)
-        bvec_file = '%s/%s.bvec' % (base_path, base_name)
-        _create_bvals_bvecs(grouped_dicoms, bval_file, bvec_file)
+            logger.info('Creating bval en bvec files')
+            bval_file = '%s/%s.bval' % (base_path, base_name)
+            bvec_file = '%s/%s.bvec' % (base_path, base_name)
+        bval, bvec = _create_bvals_bvecs(grouped_dicoms, bval_file, bvec_file)
         return {'NII_FILE': output_file,
                 'BVAL_FILE': bval_file,
-                'BVEC_FILE': bvec_file}
+                'BVEC_FILE': bvec_file,
+                'NII': nii_image,
+                'BVAL': bval,
+                'BVEC': bvec
+                }
 
-    return {'NII_FILE': output_file}
+    return {'NII_FILE': output_file,
+            'NII': nii_image}
 
 
 def _get_full_block(grouped_dicoms):
@@ -149,7 +142,7 @@ def _get_full_block(grouped_dicoms):
     # For each slice / mosaic create a data volume block
     data_blocks = []
     for index in range(0, len(grouped_dicoms)):
-        print('Creating block %s of %s' % (index + 1, len(grouped_dicoms)))
+        logger.info('Creating block %s of %s' % (index + 1, len(grouped_dicoms)))
         data_blocks.append(_timepoint_to_block(grouped_dicoms[index]))
 
     # Add the data_blocks together to one 4d block
@@ -159,6 +152,13 @@ def _get_full_block(grouped_dicoms):
     size_t = len(data_blocks)
     full_block = numpy.zeros((size_x, size_y, size_z, size_t), dtype=data_blocks[0].dtype)
     for index in range(0, size_t):
+        if full_block[:, :, :, index].shape != data_blocks[index].shape:
+            logger.warning('Missing slices (slice count mismatch between timepoint %s and %s)' % (index - 1, index))
+            logger.warning('---------------------------------------------------------')
+            logger.warning(full_block[:, :, :, index].shape)
+            logger.warning(data_blocks[index].shape)
+            logger.warning('---------------------------------------------------------')
+            raise ConversionError("MISSING_DICOM_FILES")
         full_block[:, :, :, index] = data_blocks[index]
 
     return full_block
@@ -185,21 +185,30 @@ def _get_grouped_dicoms(dicom_input):
     # now group per stack
     grouped_dicoms = [[]]  # list with first element a list
     stack_index = 0
-    previous_stack_position = -1
 
-    # loop over all sorted dicoms
-    stack_position_tag = Tag(0x0020, 0x9057)  # put this there as this is a slow step and used a lot
-    for index in range(0, len(dicoms)):
-        dicom_ = dicoms[index]
+    # loop over all sorted dicoms and sort them by stack
+    # for this we use the position and direction of the slices so we can detect a new stack easily
+    previous_position = None
+    previous_direction = None
+    for dicom_ in dicoms:
+        current_direction = None
         # if the stack number decreases we moved to the next stack
-        stack_position = 0
-        if stack_position_tag in dicom_:
-            stack_position = dicom_[stack_position_tag].value
-        if previous_stack_position > stack_position:
+        if previous_position is not None:
+            current_direction = numpy.array(dicom_.ImagePositionPatient) - previous_position
+            current_direction = current_direction / numpy.linalg.norm(current_direction)
+        if current_direction is not None and \
+                previous_direction is not None and \
+                not numpy.allclose(current_direction, previous_direction, rtol=0.05, atol=0.05):
+            previous_position = numpy.array(dicom_.ImagePositionPatient)
+            previous_direction = None
             stack_index += 1
+        else:
+            previous_position = numpy.array(dicom_.ImagePositionPatient)
+            previous_direction = current_direction
+
+        if stack_index >= len(grouped_dicoms):
             grouped_dicoms.append([])
         grouped_dicoms[stack_index].append(dicom_)
-        previous_stack_position = stack_position
 
     return grouped_dicoms
 
@@ -260,3 +269,5 @@ def _create_bvals_bvecs(grouped_dicoms, bval_file, bvec_file):
     # save the found bvecs to the file
     common.write_bval_file(bvals, bval_file)
     common.write_bvec_file(bvecs, bvec_file)
+
+    return bvals, bvecs
