@@ -6,74 +6,59 @@ dicom2nifti
 """
 
 from __future__ import print_function
+import dicom2nifti.patch_pydicom_encodings
+
+dicom2nifti.patch_pydicom_encodings.apply()
 
 import os
 import traceback
 
-import dicom
+import logging
 import nibabel
 import numpy
-from dicom.tag import Tag
+
+import pydicom.config as pydicom_config
+from pydicom.tag import Tag
 
 import dicom2nifti.common as common
+import dicom2nifti.settings as settings
 import dicom2nifti.convert_generic as convert_generic
 from dicom2nifti.exceptions import ConversionError
 
-dicom.config.enforce_valid_values = False
+pydicom_config.enforce_valid_values = False
+logger = logging.getLogger(__name__)
 
-
-def is_philips(dicom_input):
-    """
-    Use this function to detect if a dicom series is a philips dataset
-    :param dicom_input: directory with dicom files for 1 scan of a dicom_header
-    """
-    # read dicom header
-    header = dicom_input[0]
-
-    if 'Manufacturer' not in header or 'Modality' not in header:
-        return False  # we try generic conversion in these cases
-
-    # check if manufacturer is Philips
-    if 'PHILIPS' not in header.Manufacturer.upper():
-        return False
-
-    # check if Modality is mr
-    if header.Modality.upper() != 'MR':
-        return False
-
-    return True
-
-
-def dicom_to_nifti(dicom_input, output_file):
+def dicom_to_nifti(dicom_input, output_file=None):
     """
     This is the main dicom to nifti conversion fuction for philips images.
     As input philips images are required. It will then determine the type of images and do the correct conversion
 
     Examples: See unit test
+
     :param output_file: file path to the output nifti
     :param dicom_input: directory with dicom files for 1 scan
     """
 
-    assert is_philips(dicom_input)
+    assert common.is_philips(dicom_input)
 
-    if is_multiframe_dicom(dicom_input):
+    if common.is_multiframe_dicom(dicom_input):
         _assert_explicit_vr(dicom_input)
-        print('Found multiframe dicom')
+        logger.info('Found multiframe dicom')
         if _is_multiframe_4d(dicom_input):
-            print('Found sequence type: MULTIFRAME 4D')
+            logger.info('Found sequence type: MULTIFRAME 4D')
             return _multiframe_to_nifti(dicom_input, output_file)
 
         if _is_multiframe_anatomical(dicom_input):
-            print('Found sequence type: MULTIFRAME ANATOMICAL')
+            logger.info('Found sequence type: MULTIFRAME ANATOMICAL')
             return _multiframe_to_nifti(dicom_input, output_file)
     else:
-        print('Found singleframe dicom')
+        logger.info('Found singleframe dicom')
         grouped_dicoms = _get_grouped_dicoms(dicom_input)
         if _is_singleframe_4d(dicom_input):
-            print('Found sequence type: SINGLEFRAME 4D')
+            logger.info('Found sequence type: SINGLEFRAME 4D')
             return _singleframe_to_nifti(grouped_dicoms, output_file)
 
-    print('Assuming anatomical data')
+    logger.info('Assuming anatomical data')
     return convert_generic.dicom_to_nifti(dicom_input, output_file)
 
 
@@ -81,24 +66,10 @@ def _assert_explicit_vr(dicom_input):
     """
     Assert that explicit vr is used
     """
-    header = dicom_input[0]
-    if header.file_meta[0x0002, 0x0010].value == '1.2.840.10008.1.2':
-        raise ConversionError('IMPLICIT_VR_ENHANCED_DICOM')
-
-
-def is_multiframe_dicom(dicom_input):
-    """
-    Use this function to detect if a dicom series is a siemens 4D dataset
-    NOTE: Only the first slice will be checked so you can only provide an already sorted dicom directory
-    (containing one series)
-    :param dicom_input: directory with dicom files for 1 scan
-    """
-    # read dicom header
-    header = dicom_input[0]
-
-    if header.file_meta[0x0002, 0x0002].value == '1.2.840.10008.5.1.4.1.1.4.1':
-        return True
-    return False
+    if settings.validate_multiframe_implicit:
+        header = dicom_input[0]
+        if header.file_meta[0x0002, 0x0010].value == '1.2.840.10008.1.2':
+            raise ConversionError('IMPLICIT_VR_ENHANCED_DICOM')
 
 
 def _is_multiframe_diffusion_imaging(dicom_input):
@@ -129,7 +100,7 @@ def _is_multiframe_4d(dicom_input):
     Use this function to detect if a dicom series is a philips multiframe 4D dataset
     """
     # check if it is multi frame dicom
-    if not is_multiframe_dicom(dicom_input):
+    if not common.is_multiframe_dicom(dicom_input):
         return False
 
     header = dicom_input[0]
@@ -150,7 +121,7 @@ def _is_multiframe_anatomical(dicom_input):
     (containing one series)
     """
     # check if it is multi frame dicom
-    if not is_multiframe_dicom(dicom_input):
+    if not common.is_multiframe_dicom(dicom_input):
         return False
 
     header = dicom_input[0]
@@ -208,10 +179,10 @@ def _is_bval_type_a(grouped_dicoms):
     bvec_z_tag = Tag(0x2005, 0x10b2)
     for group in grouped_dicoms:
         if bvec_x_tag in group[0] and _is_float(common.get_fl_value(group[0][bvec_x_tag])) and \
-                        bvec_y_tag in group[0] and _is_float(common.get_fl_value(group[0][bvec_y_tag])) and \
-                        bvec_z_tag in group[0] and _is_float(common.get_fl_value(group[0][bvec_z_tag])) and \
-                        bval_tag in group[0] and _is_float(common.get_fl_value(group[0][bval_tag])) and \
-                        common.get_fl_value(group[0][bval_tag]) != 0:
+                bvec_y_tag in group[0] and _is_float(common.get_fl_value(group[0][bvec_y_tag])) and \
+                bvec_z_tag in group[0] and _is_float(common.get_fl_value(group[0][bvec_z_tag])) and \
+                bval_tag in group[0] and _is_float(common.get_fl_value(group[0][bval_tag])) and \
+                common.get_fl_value(group[0][bval_tag]) != 0:
             return True
     return False
 
@@ -248,48 +219,53 @@ def _multiframe_to_nifti(dicom_input, output_file):
     """
 
     # Read the multiframe dicom file
-    print('Read dicom file')
+    logger.info('Read dicom file')
     multiframe_dicom = dicom_input[0]
 
     # Create mosaic block
-    print('Creating data block')
+    logger.info('Creating data block')
     full_block = _multiframe_to_block(multiframe_dicom)
 
-    print('Creating affine')
+    logger.info('Creating affine')
 
     # Create the nifti header info
     affine = _create_affine_multiframe(multiframe_dicom)
-    print('Creating nifti')
+    logger.info('Creating nifti')
 
     # Convert to nifti
-    img = nibabel.Nifti1Image(full_block, affine)
+    nii_image = nibabel.Nifti1Image(full_block, affine)
     timing_parameters = multiframe_dicom.SharedFunctionalGroupsSequence[0].MRTimingAndRelatedParametersSequence[0]
     first_frame = multiframe_dicom[Tag(0x5200, 0x9230)][0]
-    common.set_tr_te(img, float(timing_parameters.RepetitionTime),
+    common.set_tr_te(nii_image, float(timing_parameters.RepetitionTime),
                      float(first_frame[0x2005, 0x140f][0].EchoTime))
 
-    print('Saving nifti to disk')
     # Save to disk
-    print('Saving nifti to disk %s' % output_file)
-    img.to_filename(output_file)
+    if output_file is not None:
+        logger.info('Saving nifti to disk %s' % output_file)
+        nii_image.to_filename(output_file)
 
     if _is_multiframe_diffusion_imaging(dicom_input):
-        # Create the bval en bvec files
-        base_path = os.path.dirname(output_file)
-        base_name = os.path.splitext(os.path.splitext(os.path.basename(output_file))[0])[0]
-        print('Creating bval en bvec files')
-        bval_file = '%s/%s.bval' % (base_path, base_name)
-        bvec_file = '%s/%s.bvec' % (base_path, base_name)
-        bval_file, bvec_file = _create_bvals_bvecs(multiframe_dicom, bval_file, bvec_file, img, output_file)
+        bval_file = None
+        bvec_file = None
+        if output_file is not None:
+            # Create the bval en bvec files
+            base_path = os.path.dirname(output_file)
+            base_name = os.path.splitext(os.path.splitext(os.path.basename(output_file))[0])[0]
+            logger.info('Creating bval en bvec files')
+            bval_file = '%s/%s.bval' % (base_path, base_name)
+            bvec_file = '%s/%s.bvec' % (base_path, base_name)
+        bval, bvec, bval_file, bvec_file = _create_bvals_bvecs(multiframe_dicom, bval_file, bvec_file, nii_image,
+                                                               output_file)
 
-        result = {'NII_FILE': output_file}
-        if bval_file is not None:
-            result['BVAL_FILE'] = bval_file
-        if bvec_file is not None:
-            result['BVEC_FILE'] = bvec_file
-        return result
+        return {'NII_FILE': output_file,
+                'BVAL_FILE': bval_file,
+                'BVEC_FILE': bvec_file,
+                'NII': nii_image,
+                'BVAL': bval,
+                'BVEC': bvec}
 
-    return {'NII_FILE': output_file}
+    return {'NII_FILE': output_file,
+            'NII': nii_image}
 
 
 def _singleframe_to_nifti(grouped_dicoms, output_file):
@@ -298,41 +274,49 @@ def _singleframe_to_nifti(grouped_dicoms, output_file):
     """
 
     # Create mosaic block
-    print('Creating data block')
+    logger.info('Creating data block')
     full_block = _singleframe_to_block(grouped_dicoms)
 
-    print('Creating affine')
+    logger.info('Creating affine')
     # Create the nifti header info
     affine = common.create_affine(grouped_dicoms[0])
 
-    print('Creating nifti')
+    logger.info('Creating nifti')
     # Convert to nifti
-    img = nibabel.Nifti1Image(full_block, affine)
-    common.set_tr_te(img, float(grouped_dicoms[0][0].RepetitionTime), float(grouped_dicoms[0][0].EchoTime))
+    nii_image = nibabel.Nifti1Image(full_block, affine)
+    common.set_tr_te(nii_image, float(grouped_dicoms[0][0].RepetitionTime), float(grouped_dicoms[0][0].EchoTime))
 
-    print('Saving nifti to disk %s' % output_file)
-    # Save to disk
-    img.to_filename(output_file)
+    if output_file is not None:
+        # Save to disk
+        logger.info('Saving nifti to disk %s' % output_file)
+        nii_image.to_filename(output_file)
 
     if _is_singleframe_diffusion_imaging(grouped_dicoms):
+        bval_file = None
+        bvec_file = None
         # Create the bval en bvec files
-        base_name = os.path.splitext(output_file)[0]
-        if base_name.endswith('.nii'):
-            base_name = os.path.splitext(base_name)[0]
+        if output_file is not None:
+            base_name = os.path.splitext(output_file)[0]
+            if base_name.endswith('.nii'):
+                base_name = os.path.splitext(base_name)[0]
 
-        print('Creating bval en bvec files')
-        bval_file = '%s.bval' % base_name
-        bvec_file = '%s.bvec' % base_name
-        bval_file, bvec_file = _create_singleframe_bvals_bvecs(grouped_dicoms, bval_file, bvec_file, img, output_file)
+            logger.info('Creating bval en bvec files')
+            bval_file = '%s.bval' % base_name
+            bvec_file = '%s.bvec' % base_name
+        nii_image, bval, bvec, bval_file, bvec_file = _create_singleframe_bvals_bvecs(grouped_dicoms,
+                                                                                      bval_file,
+                                                                                      bvec_file,
+                                                                                      nii_image,
+                                                                                      output_file)
 
-        result = {'NII_FILE': output_file}
-        if bval_file is not None:
-            result['BVAL_FILE'] = bval_file
-        if bvec_file is not None:
-            result['BVEC_FILE'] = bvec_file
-        return result
-
-    return {'NII_FILE': output_file}
+        return {'NII_FILE': output_file,
+                'BVAL_FILE': bval_file,
+                'BVEC_FILE': bvec_file,
+                'NII': nii_image,
+                'BVAL': bval,
+                'BVEC': bvec}
+    return {'NII_FILE': output_file,
+            'NII': nii_image}
 
 
 def _singleframe_to_block(grouped_dicoms):
@@ -342,7 +326,7 @@ def _singleframe_to_block(grouped_dicoms):
     # For each slice / mosaic create a data volume block
     data_blocks = []
     for index in range(0, len(grouped_dicoms)):
-        print('Creating block %s of %s' % (index + 1, len(grouped_dicoms)))
+        logger.info('Creating block %s of %s' % (index + 1, len(grouped_dicoms)))
         current_block = _stack_to_block(grouped_dicoms[index])
         current_block = current_block[:, :, :, numpy.newaxis]
         data_blocks.append(current_block)
@@ -372,9 +356,11 @@ def _get_grouped_dicoms(dicom_input):
 
     fast_read = True will only read the headers not the data
     """
-    # Order all dicom files by InstanceNumber
-    dicoms = sorted(dicom_input, key=lambda x: x.InstanceNumber)
-
+    # if all dicoms have an instance number try sorting by instance number else by position
+    if [d for d in dicom_input if 'InstanceNumber' in d]:
+        dicoms = sorted(dicom_input, key=lambda x: x.InstanceNumber)
+    else:
+        dicoms = common.sort_dicoms(dicom_input)
     # now group per stack
     grouped_dicoms = [[]]  # list with first element a list
     timepoint_index = 0
@@ -407,6 +393,7 @@ def _create_affine_multiframe(multiframe_dicom):
     This will work for siemens dti and 4D if in mosaic format
     """
     first_frame = multiframe_dicom[Tag(0x5200, 0x9230)][0]
+    last_frame = multiframe_dicom[Tag(0x5200, 0x9230)][-1]
     # Create affine matrix (http://nipy.sourceforge.net/nibabel/dicom/dicom_orientation.html#dicom-slice-affine)
     image_orient1 = numpy.array(first_frame.PlaneOrientationSequence[0].ImageOrientationPatient)[0:3].astype(float)
     image_orient2 = numpy.array(first_frame.PlaneOrientationSequence[0].ImageOrientationPatient)[3:6].astype(float)
@@ -417,12 +404,15 @@ def _create_affine_multiframe(multiframe_dicom):
     delta_c = float(first_frame[0x2005, 0x140f][0].PixelSpacing[1])
 
     image_pos = numpy.array(first_frame.PlanePositionSequence[0].ImagePositionPatient).astype(float)
+    last_image_pos = numpy.array(last_frame.PlanePositionSequence[0].ImagePositionPatient).astype(float)
 
-    delta_s = float(multiframe_dicom.SpacingBetweenSlices)
-    return numpy.matrix(
-        [[-image_orient1[0] * delta_r, -image_orient2[0] * delta_c, -delta_s * normal[0], -image_pos[0]],
-         [-image_orient1[1] * delta_r, -image_orient2[1] * delta_c, -delta_s * normal[1], -image_pos[1]],
-         [image_orient1[2] * delta_r, image_orient2[2] * delta_c, delta_s * normal[2], image_pos[2]],
+    number_of_stack_slices = int(common.get_ss_value(multiframe_dicom[Tag(0x2001, 0x105f)][0][Tag(0x2001, 0x102d)]))
+    delta_s = abs(numpy.linalg.norm(last_image_pos - image_pos)) / (number_of_stack_slices - 1)
+
+    return numpy.array(
+        [[-image_orient1[0] * delta_c, -image_orient2[0] * delta_r, -delta_s * normal[0], -image_pos[0]],
+         [-image_orient1[1] * delta_c, -image_orient2[1] * delta_r, -delta_s * normal[1], -image_pos[1]],
+         [image_orient1[2] * delta_c, image_orient2[2] * delta_r, delta_s * normal[2], image_pos[2]],
          [0, 0, 0, 1]])
 
 
@@ -461,20 +451,8 @@ def _multiframe_to_block(multiframe_dicom):
         # apply scaling
         rescale_intercept = frame_info[slice_index].PixelValueTransformationSequence[0].RescaleIntercept
         rescale_slope = frame_info[slice_index].PixelValueTransformationSequence[0].RescaleSlope
-        private_scale_slope = 1.0
-        private_scale_intercept = 0.0
-        private_sequence_tag = Tag(0x2005, 0x140f)
-        private_scale_intercept_tag = Tag(0x2005, 0x100d)
-        private_scale_slope_tag = Tag(0x2005, 0x100e)
-        if private_sequence_tag in frame_info[slice_index]:
-            if private_scale_intercept_tag in frame_info[slice_index][private_sequence_tag][0]:
-                private_scale_intercept = frame_info[slice_index][private_sequence_tag][0][
-                    private_scale_intercept_tag].value
-            if private_scale_slope_tag in frame_info[slice_index][private_sequence_tag][0]:
-                private_scale_slope = frame_info[slice_index][private_sequence_tag][0][private_scale_slope_tag].value
         block_data = common.do_scaling(block_data,
-                                       rescale_slope, rescale_intercept,
-                                       private_scale_slope, private_scale_intercept)
+                                       rescale_slope, rescale_intercept)
         # switch to float if needed
         if block_data.dtype != data_4d.dtype:
             data_4d = data_4d.astype(block_data.dtype)
@@ -491,6 +469,7 @@ def _multiframe_to_block(multiframe_dicom):
 
     return full_block
 
+
 def _get_t_position_index(multiframe_dicom):
     # First try temporal position index itself
     if 'DimensionIndexSequence' not in multiframe_dicom:
@@ -498,29 +477,34 @@ def _get_t_position_index(multiframe_dicom):
 
     for current_index in range(len(multiframe_dicom.DimensionIndexSequence)):
         item = multiframe_dicom.DimensionIndexSequence[current_index]
-        if 'Temporal Position Index' in item.DimensionDescriptionLabel:
+        if 'DimensionDescriptionLabel' in item and \
+                'Temporal Position Index' in item.DimensionDescriptionLabel:
             return current_index
 
     # This seems to work for most dti
     for current_index in range(len(multiframe_dicom.DimensionIndexSequence)):
         item = multiframe_dicom.DimensionIndexSequence[current_index]
-        if 'Diffusion Gradient Orientation' in item.DimensionDescriptionLabel:
+        if 'DimensionDescriptionLabel' in item and \
+                'Diffusion Gradient Orientation' in item.DimensionDescriptionLabel:
             return current_index
 
     # This seems to work for 3D grace sequences
     for current_index in range(len(multiframe_dicom.DimensionIndexSequence)):
         item = multiframe_dicom.DimensionIndexSequence[current_index]
-        if 'Effective Echo Time' in item.DimensionDescriptionLabel:
+        if 'DimensionDescriptionLabel' in item and \
+                'Effective Echo Time' in item.DimensionDescriptionLabel:
             return current_index
 
     # First try trigger delay time (inspired by http://www.dclunie.com/papers/SCAR_20040522_CTMRMF.pdf)
     for current_index in range(len(multiframe_dicom.DimensionIndexSequence)):
         item = multiframe_dicom.DimensionIndexSequence[current_index]
-        if 'Trigger Delay Time' in item.DimensionDescriptionLabel:
+        if 'DimensionDescriptionLabel' in item and \
+                'Trigger Delay Time' in item.DimensionDescriptionLabel:
             return current_index
 
     return None
 
+
 def _create_bvals_bvecs(multiframe_dicom, bval_file, bvec_file, nifti, nifti_file):
     """
     Write the bvals from the sorted dicom files to a bval file
@@ -546,14 +530,16 @@ def _create_bvals_bvecs(multiframe_dicom, bval_file, bvec_file, nifti, nifti_fil
     nifti, bvals, bvecs = _fix_diffusion_images(bvals, bvecs, nifti, nifti_file)
 
     # save the found bvecs to the file
-    if not numpy.allclose(bvals, 0) or not numpy.allclose(bvecs, 0):
+    if numpy.count_nonzero(bvals) > 0 or numpy.count_nonzero(bvecs) > 0:
         common.write_bval_file(bvals, bval_file)
         common.write_bvec_file(bvecs, bvec_file)
     else:
         bval_file = None
         bvec_file = None
+        bvals = None
+        bvecs = None
 
-    return bval_file, bvec_file
+    return bvals, bvecs, bval_file, bvec_file
 
 
 def _fix_diffusion_images(bvals, bvecs, nifti, nifti_file):
@@ -562,7 +548,7 @@ def _fix_diffusion_images(bvals, bvecs, nifti, nifti_file):
     This is sometimes added at the end by philips
     """
     # if all zero continue of if the last bvec is not all zero continue
-    if numpy.allclose(bvecs, 0) or not numpy.allclose(bvecs[-1], 0):
+    if numpy.count_nonzero(bvecs) == 0 or not numpy.count_nonzero(bvals[-1]) == 0:
         # nothing needs to be done here
         return nifti, bvals, bvecs
     # remove last elements from bvals and bvecs
@@ -570,7 +556,7 @@ def _fix_diffusion_images(bvals, bvecs, nifti, nifti_file):
     bvecs = bvecs[:-1]
 
     # remove last elements from the nifti
-    new_nifti = nibabel.Nifti1Image(nifti.get_data()[:, :, :, :-1], nifti.get_affine())
+    new_nifti = nibabel.Nifti1Image(nifti.get_data()[:, :, :, :-1], nifti.affine)
     new_nifti.to_filename(nifti_file)
 
     return new_nifti, bvals, bvecs
@@ -607,10 +593,12 @@ def _create_singleframe_bvals_bvecs(grouped_dicoms, bval_file, bvec_file, nifti,
     nifti, bvals, bvecs = _fix_diffusion_images(bvals, bvecs, nifti, nifti_file)
 
     # save the found bvecs to the file
-    if not numpy.allclose(bvals, 0) or not numpy.allclose(bvecs, 0):
+    if numpy.count_nonzero(bvals) > 0 or numpy.count_nonzero(bvecs) > 0:
         common.write_bval_file(bvals, bval_file)
         common.write_bvec_file(bvecs, bvec_file)
     else:
         bval_file = None
         bvec_file = None
-    return bval_file, bvec_file
+        bvals = None
+        bvecs = None
+    return nifti, bvals, bvecs, bval_file, bvec_file
