@@ -6,19 +6,26 @@ dicom2nifti
 """
 
 from __future__ import print_function
+import dicom2nifti.patch_pydicom_encodings
+
+dicom2nifti.patch_pydicom_encodings.apply()
 
 import os
 import re
 import traceback
 
+import logging
 import nibabel
 import numpy
-from dicom.tag import Tag
+
+from pydicom.tag import Tag
 
 import dicom2nifti.common as common
 import dicom2nifti.convert_generic as convert_generic
 from dicom2nifti.exceptions import ConversionValidationError, ConversionError
 
+logger = logging.getLogger(__name__)
+
 
 # Disable this warning as there is not reason for an init class in an enum
 # pylint: disable=w0232, r0903, E1101
@@ -35,48 +42,27 @@ class MosaicType(object):
 # pylint: enable=w0232, r0903
 
 
-def is_siemens(dicom_input):
-    """
-    Use this function to detect if a dicom series is a siemens dataset
-    :param dicom_input: directory with dicom files for 1 scan
-    """
-    # read dicom header
-    header = dicom_input[0]
-
-    # check if manufacturer is Siemens
-    if 'Manufacturer' not in header or 'Modality' not in header:
-        return False  # we try generic conversion in these cases
-
-    if 'SIEMENS' not in header.Manufacturer.upper():
-        return False
-
-    # check if Modality is mr
-    if header.Modality.upper() != 'MR':
-        return False
-
-    return True
-
-
-def dicom_to_nifti(dicom_input, output_file):
+def dicom_to_nifti(dicom_input, output_file=None):
     """
     This is the main dicom to nifti conversion function for ge images.
     As input ge images are required. It will then determine the type of images and do the correct conversion
+
     :param output_file: filepath to the output nifti
     :param dicom_input: directory with dicom files for 1 scan
     """
 
-    assert is_siemens(dicom_input)
+    assert common.is_siemens(dicom_input)
 
     if _is_4d(dicom_input):
-        print('Found sequence type: MOSAIC 4D')
+        logger.info('Found sequence type: MOSAIC 4D')
         return _mosaic_4d_to_nifti(dicom_input, output_file)
 
     grouped_dicoms = _classic_get_grouped_dicoms(dicom_input)
     if _is_classic_4d(grouped_dicoms):
-        print('Found sequence type: CLASSIC 4D')
+        logger.info('Found sequence type: CLASSIC 4D')
         return _classic_4d_to_nifti(grouped_dicoms, output_file)
 
-    print('Assuming anatomical data')
+    logger.info('Assuming anatomical data')
     return convert_generic.dicom_to_nifti(dicom_input, output_file)
 
 
@@ -93,7 +79,7 @@ def _is_mosaic(dicom_input):
         header = dicom_input[0]
 
     # check if image type contains m and mosaic
-    if 'MOSAIC' not in header.ImageType:
+    if 'ImageType' not in header or 'MOSAIC' not in header.ImageType:
         return False
 
     if 'AcquisitionMatrix' not in header or header.AcquisitionMatrix is None:
@@ -151,41 +137,49 @@ def _mosaic_4d_to_nifti(dicom_input, output_file):
     http://slicer.org/doc/html/DICOMDiffusionVolumePlugin_8py_source.html
     """
     # Get the sorted mosaics
-    print('Sorting dicom slices')
+    logger.info('Sorting dicom slices')
     sorted_mosaics = _get_sorted_mosaics(dicom_input)
     common.validate_orientation(sorted_mosaics)
 
     # Create mosaic block
-    print('Creating data block')
+    logger.info('Creating data block')
     full_block = _mosaic_get_full_block(sorted_mosaics)
 
-    print('Creating affine')
+    logger.info('Creating affine')
     # Create the nifti header info
     affine = _create_affine_siemens_mosaic(dicom_input)
-    print('Creating nifti')
+    logger.info('Creating nifti')
     # Convert to nifti
-    img = nibabel.Nifti1Image(full_block, affine)
-    common.set_tr_te(img, float(sorted_mosaics[0].RepetitionTime), float(sorted_mosaics[0].EchoTime))
-    print('Saving nifti to disk')
+    nii_image = nibabel.Nifti1Image(full_block, affine)
+    common.set_tr_te(nii_image, float(sorted_mosaics[0].RepetitionTime), float(sorted_mosaics[0].EchoTime))
+    logger.info('Saving nifti to disk')
     # Save to disk
-    img.to_filename(output_file)
+    if output_file is not None:
+        nii_image.to_filename(output_file)
 
     if _is_diffusion_imaging(dicom_input[0]):
         # Create the bval en bvec files
-        print('Creating bval en bvec files')
-        base_path = os.path.dirname(output_file)
-        base_name = os.path.splitext(os.path.splitext(os.path.basename(output_file))[0])[0]
-        print('Creating bval en bvec files')
-        bval_file = '%s/%s.bval' % (base_path, base_name)
-        bvec_file = '%s/%s.bvec' % (base_path, base_name)
-        _create_bvals(sorted_mosaics, bval_file)
-        _create_bvecs(sorted_mosaics, bvec_file)
+        logger.info('Creating bval en bvec')
+        bval_file = None
+        bvec_file = None
+        if output_file is not None:
+            base_path = os.path.dirname(output_file)
+            base_name = os.path.splitext(os.path.splitext(os.path.basename(output_file))[0])[0]
+            logger.info('Saving bval en bvec files')
+            bval_file = '%s/%s.bval' % (base_path, base_name)
+            bvec_file = '%s/%s.bvec' % (base_path, base_name)
+        bvals = _create_bvals(sorted_mosaics, bval_file)
+        bvecs = _create_bvecs(sorted_mosaics, bvec_file)
 
         return {'NII_FILE': output_file,
                 'BVAL_FILE': bval_file,
-                'BVEC_FILE': bvec_file}
+                'BVEC_FILE': bvec_file,
+                'NII': nii_image,
+                'BVAL': bvals,
+                'BVEC': bvecs}
 
-    return {'NII_FILE': output_file}
+    return {'NII_FILE': output_file,
+            'NII': nii_image}
 
 
 def _classic_4d_to_nifti(grouped_dicoms, output_file):
@@ -199,38 +193,45 @@ def _classic_4d_to_nifti(grouped_dicoms, output_file):
     common.validate_orientation(all_dicoms)
 
     # Create mosaic block
-    print('Creating data block')
+    logger.info('Creating data block')
     full_block = _classic_get_full_block(grouped_dicoms)
 
-    print('Creating affine')
+    logger.info('Creating affine')
     # Create the nifti header info
     affine = common.create_affine(grouped_dicoms[0])
 
-    print('Creating nifti')
+    logger.info('Creating nifti')
     # Convert to nifti
-    img = nibabel.Nifti1Image(full_block, affine)
-    common.set_tr_te(img, float(grouped_dicoms[0][0].RepetitionTime), float(grouped_dicoms[0][0].EchoTime))
-    print('Saving nifti to disk')
+    nii_image = nibabel.Nifti1Image(full_block, affine)
+    common.set_tr_te(nii_image, float(grouped_dicoms[0][0].RepetitionTime), float(grouped_dicoms[0][0].EchoTime))
+    logger.info('Saving nifti to disk')
     # Save to disk
-    img.to_filename(output_file)
+    if output_file is not None:
+        nii_image.to_filename(output_file)
 
     if _is_diffusion_imaging(grouped_dicoms[0][0]):
-        # Create the bval en bvec files
-        print('Creating bval en bvec files')
-        base_path = os.path.dirname(output_file)
-        base_name = os.path.splitext(os.path.splitext(os.path.basename(output_file))[0])[0]
-        print('Creating bval en bvec files')
-        bval_file = '%s/%s.bval' % (base_path, base_name)
-        bvec_file = '%s/%s.bvec' % (base_path, base_name)
+        logger.info('Creating bval en bvec')
+        bval_file = None
+        bvec_file = None
+        if output_file is not None:
+            base_path = os.path.dirname(output_file)
+            base_name = os.path.splitext(os.path.splitext(os.path.basename(output_file))[0])[0]
+            logger.info('Creating bval en bvec files')
+            bval_file = '%s/%s.bval' % (base_path, base_name)
+            bvec_file = '%s/%s.bvec' % (base_path, base_name)
 
-        _create_bvals(grouped_dicoms, bval_file)
-        _create_bvecs(grouped_dicoms, bvec_file)
+        bval = _create_bvals(grouped_dicoms, bval_file)
+        bvec = _create_bvecs(grouped_dicoms, bvec_file)
 
         return {'NII_FILE': output_file,
                 'BVAL_FILE': bval_file,
-                'BVEC_FILE': bvec_file}
+                'BVEC_FILE': bvec_file,
+                'NII': nii_image,
+                'BVAL': bval,
+                'BVEC': bvec}
 
-    return {'NII_FILE': output_file}
+    return {'NII_FILE': output_file,
+            'NII': nii_image}
 
 
 def _classic_get_grouped_dicoms(dicom_input):
@@ -241,7 +242,10 @@ def _classic_get_grouped_dicoms(dicom_input):
     """
     # Loop overall files and build dict
     # Order all dicom files by InstanceNumber
-    dicoms = sorted(dicom_input, key=lambda x: x.InstanceNumber)
+    if [d for d in dicom_input if 'InstanceNumber' in d]:
+        dicoms = sorted(dicom_input, key=lambda x: x.InstanceNumber)
+    else:
+        dicoms = common.sort_dicoms(dicom_input)
 
     # now group per stack
     grouped_dicoms = []
@@ -268,7 +272,7 @@ def _classic_get_full_block(grouped_dicoms):
     # For each slice / mosaic create a data volume block
     data_blocks = []
     for index in range(0, len(grouped_dicoms)):
-        print('Creating block %s of %s' % (index + 1, len(grouped_dicoms)))
+        logger.info('Creating block %s of %s' % (index + 1, len(grouped_dicoms)))
         data_blocks.append(_classic_timepoint_to_block(grouped_dicoms[index]))
 
     # Add the data_blocks together to one 4d block
@@ -409,15 +413,10 @@ def _mosaic_to_block(mosaic):
     number_x = int(mosaic.Rows / size[0])
     number_y = int(mosaic.Columns / size[1])
 
-    # get the format
-    format_string = common.get_numpy_type(mosaic)
-
-    # reshape the 1d array to 3d
-    data_1d = numpy.fromstring(mosaic.PixelData, dtype=format_string)
     # recreate 2d slice
-    data_2d = numpy.reshape(data_1d, (size[1] * number_y, size[0] * number_x))
+    data_2d = mosaic.pixel_array
     # create 3d block
-    data_3d = numpy.zeros((size[2], size[1], size[0]), dtype=format_string)
+    data_3d = numpy.zeros((size[2], size[1], size[0]), dtype=data_2d.dtype)
     # fill 3d block by taking the correct portions of the slice
     z_index = 0
     for y_index in range(0, number_y):
@@ -426,10 +425,10 @@ def _mosaic_to_block(mosaic):
         for x_index in range(0, number_x):
             if mosaic_type == MosaicType.ASCENDING:
                 data_3d[z_index, :, :] = data_2d[size[1] * y_index:size[1] * (y_index + 1),
-                                                 size[0] * x_index:size[0] * (x_index + 1)]
+                                         size[0] * x_index:size[0] * (x_index + 1)]
             else:
                 data_3d[size[2] - (z_index + 1), :, :] = data_2d[size[1] * y_index:size[1] * (y_index + 1),
-                                                                 size[0] * x_index:size[0] * (x_index + 1)]
+                                                         size[0] * x_index:size[0] * (x_index + 1)]
             z_index += 1
             if z_index >= size[2]:
                 break
@@ -459,10 +458,10 @@ def _create_affine_siemens_mosaic(dicom_input):
     image_pos = dicom_header.ImagePositionPatient
 
     delta_s = dicom_header.SpacingBetweenSlices
-    return numpy.matrix(
-        [[-image_orient1[0] * delta_r, -image_orient2[0] * delta_c, -delta_s * normal[0], -image_pos[0]],
-         [-image_orient1[1] * delta_r, -image_orient2[1] * delta_c, -delta_s * normal[1], -image_pos[1]],
-         [image_orient1[2] * delta_r, image_orient2[2] * delta_c, delta_s * normal[2], image_pos[2]],
+    return numpy.array(
+        [[-image_orient1[0] * delta_c, -image_orient2[0] * delta_r, -delta_s * normal[0], -image_pos[0]],
+         [-image_orient1[1] * delta_c, -image_orient2[1] * delta_r, -delta_s * normal[1], -image_pos[1]],
+         [image_orient1[2] * delta_c, image_orient2[2] * delta_r, delta_s * normal[2], image_pos[2]],
          [0, 0, 0, 1]])
 
 
@@ -480,6 +479,7 @@ def _create_bvals(sorted_dicoms, bval_file):
         bvals.append(common.get_is_value(dicom_headers[Tag(0x0019, 0x100c)]))
     # save the found bvecs to the file
     common.write_bval_file(bvals, bval_file)
+    return numpy.array(bvals)
 
 
 def _create_bvecs(sorted_dicoms, bvec_file):
@@ -529,5 +529,6 @@ def _create_bvecs(sorted_dicoms, bvec_file):
             # normalize the bvec
             new_bvec /= numpy.linalg.norm(new_bvec)
         bvecs[index, :] = new_bvec
-    # save the found bvecs to the file
-    common.write_bvec_file(bvecs, bvec_file)
+        # save the found bvecs to the file
+        common.write_bvec_file(bvecs, bvec_file)
+    return numpy.array(bvecs)
